@@ -15,16 +15,11 @@
 #
 
 from gi.repository import Gio
-
-import gettext
-from gettext import gettext as _
-gettext.textdomain('pasaffe')
-
 import logging
-logger = logging.getLogger('pasaffe')
-
 from pasaffe_lib.PreferencesDialog import PreferencesDialog
 
+logger = logging.getLogger('pasaffe')
+
 
 class PreferencesPasaffeDialog(PreferencesDialog):
     __gtype_name__ = "PreferencesPasaffeDialog"
@@ -56,5 +51,11 @@ class PreferencesPasaffeDialog(PreferencesDialog):
         widget = self.builder.get_object('tree-expansion')
         settings.bind("tree-expansion", widget, "active-id",
                       Gio.SettingsBindFlags.DEFAULT)
+        widget = self.builder.get_object('double-click')
+        settings.bind("double-click", widget, "active-id",
+                      Gio.SettingsBindFlags.DEFAULT)
+        widget = self.builder.get_object('display-usernames')
+        settings.bind("display-usernames", widget, "active",
+                      Gio.SettingsBindFlags.DEFAULT)
 
         # Code for other initialization actions should be added here.
