@@ -17,12 +17,10 @@
 
 import sys
 import os.path
-import unittest
-import time
 sys.path.insert(0, os.path.realpath(os.path.join(os.path.dirname(__file__),
                                                  "..")))
 
-from pasaffe_lib.readdb import PassSafeFile
+from pasaffe_lib.readdb import PassSafeFile  # noqa: E402
 
 
 def get_test_data():
@@ -67,6 +65,7 @@ def create_test_db():
 
     return passfile
 
+
 if __name__ == '__main__':
     # This script can be used to create a test database
     # Simply run it with a filename, and a database will be created
